#version 430

layout(location = 0) out vec4 fragColor;

in vec3 n; 
in vec3 v;
//in vec3 p;
in vec3 color;
in vec3 l[20]; // direction to light
in vec4 fragPosLightSpace;


uniform sampler2D shadowMap;


uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;
uniform mat4 lightSpace;

uniform vec3 lightPosition[20];
uniform vec3 direction[20];
uniform float coneAngle[20];
uniform uint type[20];
uniform float constant[20];
uniform float linear[20];
uniform float quadratic[20];
uniform vec3 ambientColor[20];
uniform vec3 diffuseColor[20];
uniform vec3 specularColor[20];
uniform int actualLights;


uniform vec3 cameraLocation;

float shininess = 0.5f;

float calcShadow(vec4 fragPosLightSpace, float lightDirNormalAngle){
	//perspective divide
	vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w; //Range = [-1,1]

	//Map to range [0,1]
	projCoords = projCoords * 0.5 + 0.5; 

	//Retrieve depth value of the current position from shadowmap 
	//float closestDepth = texture(shadowMap, projCoords.xy).r;   

	//retrieve depth value of this current point
	float currentDepth = projCoords.z;

	//Shadow bias
	//float bias = 0.005;
	float bias = max(0.05 * (1.0 - lightDirNormalAngle), 0.005);  

    // check whether current frag pos is in shadow
    float shadow = 0.0; 

	if(projCoords.z <= 1.0f){
		//Groesse eines texels auf der shadowmap mit Mipmaplevel 0.
		vec2 texelSize = 1.0/ textureSize(shadowMap,0); // wird benutzt um in der shadowMap zu navigieren

		//Tiefenwerte von 9 Positionen auslesen und Mittelwert bilden
		for(int x = -2; x <= 2; ++x){
			for(int y = -2; y <= 2; ++y){
				float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 

				 // check whether current frag pos is in shadow
				shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
			}    
		}
		shadow = shadow/25.0f;
	}

    return shadow;
}

void main(){
			
			vec3 V = normalize(v);
			vec3 N = normalize(n);
			vec3 I = vec3(1);
			vec3 diffus = vec3(0);
			vec3 spec = vec3(0);
			vec3 blinnPhong = vec3(0);
			vec3 lambert = vec3(0);
			vec3 coneDirection = vec3(0);
			float cosThetaSpec;
			float cosThetaDiff;
			float cosTheta;
			float specularStrenght = 2.0f;
			float diffusStrenght = 4;
			float centerLightToSurfaceAngle = 0;

			vec3 ambient = vec3(0.2f,0.2f,0.2f);
			//vec3 vDir = normalize(cameraLocation - p);
			float shadow = 0;


			for(int i = 0; i < actualLights; i++){
				

					vec3 L = normalize(l[i]);

					//coneDirection = normalize(direction[i]);
					// Get the angle between the center of the cone and the ray of light.
				//	centerLightToSurfaceAngle = degrees(acos(dot(-L, coneDirection)));
				//	if(centerLightToSurfaceAngle < coneAngle[i]){

						vec3 H = normalize(L + V);
						//vec3 reflection = reflect(-L, N);
						cosThetaSpec = pow(max(dot(N, H), 0.0), 32);

						spec = cosThetaSpec * specularColor[i] * specularStrenght;

						//Diffuser Teil
						cosThetaDiff = max(dot(L, N), 0.0);
						diffus =  cosThetaDiff * diffuseColor[i] * diffusStrenght ;

						/*float surfaceLightDistance = distance( lightPos  , p );

						float a = constant[i] + surfaceLightDistance * linear[i] + pow(surfaceLightDistance,2) * quadratic[i];
				
						vec3 ambient = ambientColor[i];
						vec3 diffuse = diffuseColor[i]; 
						vec3 specular = specularColor[i];
						I = ( (1/a) * ambient  +  (1/a) * diffuse + (1/a) * specular ) * 1/3;
						*/
						//vec3 res = (ambient+ diffuse + specular) * color

						//Schatten
						if(type[i] == 1){
						shadow = calcShadow(fragPosLightSpace, dot(L,N) );
						}
						if(type[i] == 2){
						shadow = 0;
						}

						blinnPhong = blinnPhong +  ((ambientColor[i] + (1.0 - shadow) *( diffus + spec)) * color );
					//}
			
			}
			fragColor = vec4(blinnPhong,1.0f);
	

}




/*
				//lambert
				vec3 lightPos = lightPosition[i];
				cosTheta = max(dot(N, normalize(lightPos - p)),0);
				float surfaceLightDistance = distance( lightPos  , p );

				float a = constant[i] + surfaceLightDistance * linear[i] + pow(surfaceLightDistance,2) * quadratic[i];
				
				vec3 ambient = ambientColor[i];
				vec3 diffuse = diffuseColor[i]; 
				vec3 specular = specularColor[i];
				//I = ( (1/a) * ambient  +  (1/a) * diffuse + (1/a) * specular ) * 1/3;
				lambert = lambert + (  ( color * I) * cosTheta );
				*/





		//fragColor =vec4(lambert,1.0f);
			/*

			// blinn = Summe von [ (Fr alle Lichtquellen) I *( spec + diffus(= lambert)) ]
        // Ispec = ks * cosTheta
        // cosTheta = Winkelhalbierende = (V + L / ||V + L||)^shininess
        // V = Punkt nach Betrachter normalisiert
        // L = Punkt nach Licht normalisiert
        // ks = Spekulrer Reflexionskoeffizient (0 bis 1)
        V = V.normalize();
        N = N.normalize();
        Vec3 I;
        Vec3 diffus;
        Vec3 spec;
        Vec3 blinnPhong = Vec3.zero;
        float cosThetaSpec;
        float cosThetaDiff;
        for (Light light : lights) {
            Vec3 L = light.calcDirection(P);

            //Spekular Teil
            Vec3 H = Vec3.add(L, V).normalize();
            cosThetaSpec = (float) Math.pow(Math.max(Vec3.dot(N, H), 0), shininess);
            spec = Vec3.mul(specularColor, cosThetaSpec);
            // multiply the specular factor with the dot product of the normalized normal and the normalized light vector.
            spec = Vec3.mul(spec, Math.max(Vec3.dot(N, L), 0));

            //Diffuser Teil
            cosThetaDiff = Math.max(Vec3.dot(L, N), 0);
            diffus = Vec3.mul(diffuseColor, cosThetaDiff);

            I = light.calcIntensity(P);
            blinnPhong = Vec3.add(blinnPhong, Vec3.mul(I, Vec3.add(diffus, spec)));


			float surfaceLightDistance = Vec3.sub(this._position, surfacePosition).length();
        float a = constantAttenuation + surfaceLightDistance * linearAttenuation + (surfaceLightDistance * surfaceLightDistance) * quadraticAttenuation;

        return Vec3.mul(intensity, 1/a);

        }

        return blinnPhong;
			*/