#version 430

layout(location = 0) out vec4 fragColor;


in vec2 texCoords;
in vec3 n; 
in vec3 v;
in vec3 l[20]; // direction to light
//in vec3 lDir[20];
in vec4 fragPosLightSpace;
in mat3 TBN;


uniform bool normalMapping;

uniform sampler2D myTexture;
uniform sampler2D shadowMap;
uniform sampler2D normalMap;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;
uniform mat4 lightSpace;


uniform vec3 lightPosition[20];
uniform vec3 direction[20];
uniform float coneAngle[20];
uniform uint type[20];
uniform float constant[20];
uniform float linear[20];
uniform float quadratic[20];
uniform vec3 ambientColor[20];
uniform vec3 diffuseColor[20];
uniform vec3 specularColor[20];
uniform int actualLights;


uniform vec3 cameraLocation;

float shininess = 32.0f;
 

float calcShadow(vec4 fragPosLightSpace, float lightDirNormalAngle){
	//perspective divide
	vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w; //Range = [-1,1]

	//Map to range [0,1]
	projCoords = projCoords * 0.5 + 0.5; 

	//Retrieve depth value of the current position from shadowmap 
	//float closestDepth = texture(shadowMap, projCoords.xy).r;   

	//retrieve depth value of this current point
	float currentDepth = projCoords.z;

	//Shadow bias
	//float bias = 0.005;
	float bias = max(0.05 * (1.0 - lightDirNormalAngle), 0.005);  


   // check whether current frag pos is in shadow
    float shadow = 0.0; 

	if(projCoords.z <= 1.0f){
		//Groesse eines texels auf der shadowmap mit Mipmaplevel 0.
		vec2 texelSize = 1.0/ textureSize(shadowMap,0); // wird benutzt um in der shadowMap zu navigieren

		//Tiefenwerte von 9 Positionen auslesen und Mittelwert bilden
		for(int x = -2; x <= 2; ++x){
			for(int y = -2; y <= 2; ++y){
				float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 

				 // check whether current frag pos is in shadow
				shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
			}    
		}
		shadow = shadow/25.0f;
	}
    return shadow;
}


void main(){
		vec4 texCol = texture(myTexture, texCoords);
		//vec4 texCol = texture(normalMap, texCoords);

			float alpha = texCol.a;
			vec3 V = normalize(v);
			vec3 N = normalize(n);
			if(normalMapping){
			N = texture(normalMap, texCoords).rgb;
			N = normalize(N * 2.0 - 1.0); //Map to -1 1 -> NDC
			N = normalize(TBN * N);
			}
			vec3 I = vec3(1);
			vec3 diffus = vec3(0);
			vec3 spec = vec3(0);
			vec3 blinnPhong = vec3(0);
			vec3 lambert = vec3(0);
			vec3 coneDirection = vec3(0);
			float cosThetaSpec;
			float cosThetaDiff;
			float cosTheta;
			float specularStrenght = 1.8f;
			float diffusStrenght = 4;
			float centerLightToSurfaceAngle = 0;
			float shadow = 0;
			bool debugMode = false;
			float depthValue = texture(shadowMap, texCoords).r;
		
			vec3 ambient = vec3(0.1f,0.1f,0.1f);

			for(int i = 0; i < actualLights; i++){
				if (type[i] == 1){
					//vec3 lightPos = lightPosition[i];

					vec3 L = normalize(l[i]);
					//vec3(1,0,-5)direction[i]
					//coneDirection = normalize(direction[i]);
					// Get the angle between the center of the cone and the ray of light.
					//centerLightToSurfaceAngle = degrees(acos(dot(-L, coneDirection)));
					//if(centerLightToSurfaceAngle < coneAngle[i]){
						vec3 H = normalize(L + V);
						//vec3 reflection = reflect(-L, N);
						cosThetaSpec = pow(max(dot(V, N), 0.0), shininess);

						//Spec Teil
						spec = cosThetaSpec * specularColor[i] * specularStrenght;

						//Diffuser Teil
						cosThetaDiff = max(dot(L, N), 0.0);
						diffus = cosThetaDiff * diffuseColor[i] * diffusStrenght ;

						//Schatten
						shadow = calcShadow(fragPosLightSpace, dot(L,N) );
						
						blinnPhong = blinnPhong +  ((ambientColor[i] + (1.0 - shadow) *( diffus + spec)) * texCol.rgb);
				
				if(constant[i] < 1){
				debugMode = true;
				}else{
				debugMode = false;
				}
					//}
				
				}	
			}
			fragColor = vec4(blinnPhong, alpha);
			if(debugMode){
				fragColor = vec4(vec3(depthValue), 1.0); //+= vec4(vec3(0.0,depthValue,0.0), 1.0);
			}
			//fragColor += vec4(vec3(0.0,depthValue,0.0), 1.0);
			
}
/*	vec3 V = normalize(v);
			vec3 N = normalize(n);
			vec3 blinnPhong = vec3(0.0f,0.0f,0.0f);
			float specularStrenght = 1.5f;
			float ambientStrenght = 1;
			vec4 texCol = texture(myTexture, texCoords);
			for(int i = 0; i < actualLights; i++){
				vec3 L = normalize(l[i]);
				//Ambient part
				vec3 Iamb = ambientStrenght * ambientColor[i];

				//Diffuse part
				float diffuse = max(dot(L, N), 0.0);
				vec3 Idiffuse = diffuse * diffuseColor[i] ;

				//Specular part
				vec3 Ispec = vec3(0.0f,0.0f,0.0f);
				if(dot(N, L) > 0){
					// half vector
					vec3 H = normalize(L + V);
					float specularTerm = pow(dot(N, H), shininess);
					Ispec = specularColor[i] * specularStrenght * specularTerm;
				}
				blinnPhong = blinnPhong +  ((Iamb + Idiffuse + Ispec) * texCol.rgb);
			}
			fragColor = vec4(blinnPhong,1);
*/
		/*
*/